
import sys
import os

import subprocess as sp
from time import sleep


def hub(port=5001, detached=False, verbose=True):
    """
    Launch ezchat hub server
    In new terminal window in detached=True
    """
    name = 'hub'
    filename, msg = create_script(name=name, port=port, verbose=verbose)
    ref = run_script(filename, msg=msg, detached=detached)

    # ref = launch_web_server(name='hub', port=port)
    return ref


def client(port=5002, detached=False, verbose=True):
    """
    Launch ezchat client server
    In new terminal window if detached=True
    """
    name = 'client'
    filename, msg = create_script(name=name, port=port, verbose=verbose)
    ref = run_script(filename, msg=msg, detached=detached)

    # ref = launch_web_server(name='client', port=port)
    return ref


def create_script(name=None, port=5000, verbose=True):
    """
    Create run script for chat server or client
    """
    if name not in ['hub', 'client']:
        print('Bad arguments')
        return

    here = os.path.dirname(os.path.realpath(__file__))

    if sys.platform == 'darwin':
        command = "cd {}; export port={}; python {}.py"
        suffix = 'command'
    elif sys.platform == 'win32':
        command = "cd /d {}; SET port={}; call python {}.py; pause"
        suffix = 'bat'
    elif sys.platform == 'linux':
        command = "cd {}; export port={}; python {}.py"
        suffix = 'sh'

    filename = os.path.join(here, '{}.{}'.format(name, suffix))
    script = command.format(here, port, name)

    with open(filename, 'w') as f:
        f.write(script)

    msg = 'Open browser to http://localhost:{}'.format(port)
    if verbose:
        print('Script {} created'.format(filename))

    return filename, msg


def run_script(filename, msg=None, detached=False):
    """
    Run script filename
    In new terminal window in detached=True
    """
    here = os.path.dirname(os.path.realpath(__file__))
    name, suffix = os.path.basename(filename).split('.')

    if detached:
        if sys.platform == 'darwin' or sys.platform == 'linux':
            # launch server
            cmd = 'cd {}; chmod +x ./{}.{}; open ./{}.{}'
            cmd = cmd.format(here, name, suffix, name, suffix)
            p = sp.Popen(cmd, stdout=sp.PIPE, shell=True)

            # get process ref: pids
            sleep(0.8)
            cmd = "'python {}.py'".format(name)
            cmd = "ps aux | grep " + cmd + " | grep -v grep | awk '{print $2}'"
            p = sp.Popen(cmd, stdout=sp.PIPE, shell=True)
            res = p.communicate()[0]
            pids = res.decode('utf-8').split('\n')
            pids = [e for e in pids if e != '']
            ref = pids

        elif sys.platform == 'win32':
            # launch server
            cmd = 'cd /d {}; call {}.{}'
            p = sp.Popen([cmd.format(here, name, suffix)],
                         creationflags=sp.CREATE_NEW_CONSOLE)
            # process id is p itself - detached console is windows specific
            ref = p

        print('Web server "ezchat {}" started'.format(name))
        print('To check it is running, look for its terminal window')
        if msg:
            print(msg)
        print('Note: The server will crash upon start if the port is being used')
        return ref

    else:
        if sys.platform == 'darwin' or sys.platform == 'linux':
            # launch server
            cmd = '{}'.format(filename)
            os.system(cmd)

        elif sys.platform == 'win32':
            # launch server
            cmd = '{}'.format(filename)
            os.system(cmd)

        print('Server interrupted: {}'.format(name))
        return None


def kill(ref=None):
    """
    Kill web server identified by its ref (os specific)
    """
    if sys.platform == 'darwin' or sys.platform == 'linux':
        if not isinstance(ref, list):
            print('Wrong reference')
            return

        pids = ref
        print('Killing pids: {}'.format(pids))
        # kill pids
        cmd = 'kill {}'.format(' '.join(pids))
        # os.system(cmd)
        p = sp.Popen(cmd, stdout=sp.PIPE, shell=True)
        print('Server killed')

    elif sys.platform == 'win32':
        if not isinstance(p, sp.Popen):
            print('Wrong reference')
            return

        p = ref
        print('Killing detached console')
        # kill console
        p.kill()
        print('Server killed')





import os
import logging

import datetime as dt

import flask
from flask import Flask, render_template, redirect, url_for, request
from flask_socketio import SocketIO, join_room, leave_room, send, disconnect


logging.getLogger().setLevel(logging.DEBUG)

app = Flask(__name__, template_folder='templates')
app.config['DEBUG'] = True
app.config['SECRET'] = 'secret!'


# Set this variable to "threading", "eventlet" or "gevent" to test the
# different async modes, or leave it set to None for the application to choose
# the best option based on installed packages.
async_mode = None

socketio = SocketIO(app, json=flask.json, async_mode=async_mode)

clients = []
passwords = {}


def fmt(d):
    return d.strftime('%d%b %H:%M:%S')


@socketio.on('join', namespace='/chat')
def on_join(data):
    global clients

    print(data)
    name = data.get('name', None)
    room = data.get('room', None)
    password = data.get('password', None)

    if (not name) or (not room) or (not password):
        msg = 'Wrong join: ' + str(data)
        print(msg)
        return {'msg': msg}

    new_client = {'name': name,
                  'room': room,
                  'password': password,
                  'datetime': fmt(dt.datetime.now()),
                  'sid': request.sid}

    data = {'datetime': fmt(dt.datetime.now()),
            'sid': request.sid}

    warning_msg = None
    if name in [c['name'] for c in clients if c['room'] == room]:
        warning_msg = "Pseudo already used. It'll work but confusing. Change it !"
        data['warning_msg'] = warning_msg

    if room not in passwords:
        passwords[room] = password

    if password != passwords[room]:
        warning_msg = 'Wrong password. Messages will appear garbled if at all !'
        data['warning_msg'] = warning_msg

    join_room(room)
    clients.append(new_client)

    print('new client =', str(new_client))
    print('connected clients =', str(clients))
    print('passwords =', str(passwords))

    data2 = {'name': 'Server',
             'text': '{} has joined the room'.format(name)}
    socketio.emit('msg', data2, room=room, namespace='/chat')

    data3 = {'clients': [c for c in clients
                         if c['room'] == new_client['room']]}
    socketio.emit('status', data3, room=room, namespace='/chat')

    return data


@socketio.on('kill', namespace='/chat')
def on_kill(data):
    global clients

    print(data)
    sid = data.get('sid', None)

    if not sid:
        msg = 'Wrong kill: ' + str(data)
        print(msg)
        return {'msg': msg}

    out_client = {'sid': sid}
    out_client = [c for c in clients if c['sid'] == out_client['sid']][0]
    room = out_client['room']
    clients = [c for c in clients if c['sid'] != out_client['sid']]

    print('client killed =', str(out_client))
    print('connected clients =', str(clients))
    print('passwords =', str(passwords))

    data = {'clients': [c for c in clients if c['room'] == room]}
    socketio.emit('status', data, room=room, namespace='/chat')

    leave_room(room, sid=sid)
    disconnect()


@socketio.on('leave', namespace='/chat')
def on_leave(data):
    global clients

    print(data)
    sid = data.get('sid', None)

    if not sid:
        msg = 'Wrong leave: ' + str(data)
        print(msg)
        return {'msg': msg}

    out_client = {'sid': sid}
    out_client = [c for c in clients if c['sid'] == out_client['sid']][0]
    room = out_client['room']
    name = out_client['name']
    clients = [c for c in clients if c['sid'] != out_client['sid']]

    set_rooms = set([c['room'] for c in clients])
    if room not in set_rooms:
        passwords.pop(room)

    print('client leaving =', str(out_client))
    print('connected clients =', str(clients))
    print('passwords =', str(passwords))

    data = {'clients': [c for c in clients if c['room'] == room]}
    socketio.emit('status', data, room=room, namespace='/chat')

    data = {'name': 'Server',
            'text': '{} has left the room'.format(name)}
    socketio.emit('msg', data, room=room, namespace='/chat')

    leave_room(room)
    disconnect()


@socketio.on('clients', namespace='/chat')
def on_clients(data):

    client = {'sid': request.sid}
    client = [c for c in clients if c['sid'] == client['sid']][0]
    room = client['room']

    print('request for clients')
    print('connected clients =', str(clients))
    print('passwords =', str(passwords))

    data = {'clients': [c for c in clients if c['room'] == room]}
    socketio.emit('status', data, room=room, namespace='/chat')

    return data


@socketio.on('msg', namespace='/chat')
def on_msg(data):
    print(data)
    name = data.get('name', None)
    room = data.get('room', None)
    text = data.get('text', None)
    sid = data.get('sid', None)

    if (not name) or (not room) or (not text) or (not sid):
        msg = 'Wrong msg:' + str(data)
        print(msg)
        return {'msg': msg}

    data = {'room': room,
            'name': name,
            'sid': sid,
            'text': text,
            }
    socketio.emit('msg', data, room=room,
                  namespace='/chat', include_self=False)
    msg = 'msg "{}" from {} broadcasted to room {}'.format(text, name, room)
    return {'msg': msg}


# default route
@app.route('/info', methods=['GET'])
def info():
    """single page app"""
    return render_template('info.html', async_mode=socketio.async_mode)


# fallback
@app.route('/')
def root():
    """redirect"""
    return redirect(url_for('info'))


@app.route('/<path:dummy>')
def fallback(dummy):
    """redirect"""
    return redirect(url_for('info'))


port = int(os.environ.get('port', 5000))

logging.info(app.config)
logging.info('Starting ezchat hub')

ssl_context = (os.path.join('ssl', 'host.cert'), os.path.join('ssl', 'host.key'))
# app.run(host='0.0.0.0', port=port, debug=True, threaded=False, ssl_context=ssl_context)
socketio.run(app, host='0.0.0.0', port=port, debug=True, threaded=False, ssl_context=ssl_context)




import os
import logging

import flask
from flask import Flask, render_template, redirect, url_for, send_from_directory


logging.getLogger().setLevel(logging.DEBUG)


app = Flask(__name__, static_folder='./chatbox/dist')
app.config['DEBUG'] = True



# default route
@app.route('/', methods=['GET'])
def root():
    """single page app"""
    return app.send_static_file('index.html')

@app.route('/<path:path>')
def send_asset(path):
    """All other static assets"""
    return app.send_static_file(path)


port = int(os.environ.get('port', 5001))
# print(port)

logging.info(app.config)
logging.info('Starting ezchat client')
app.run(debug=True, port=port)



# necessary to push to PyPI
# cf. http://peterdowns.com/posts/first-time-with-pypi.html
# cf. https://tom-christie.github.io/articles/pypi/
# cf. https://pythonhosted.org/setuptools/setuptools.html
# cf. http://peterdowns.com/posts/first-time-with-pypi.html


from setuptools import setup
from distutils.util import convert_path

main_ns = {}
ver_path = convert_path('ezchat/__meta__.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

with open('README.rst') as f:
    long_description = f.read()

name = main_ns['__name__']
version = main_ns['__version__']


setup(
  name = name,
  packages = [name],
  version = version,
  description = 'easy instant chat webapp',
  long_description = long_description,
  author = 'oscar6echo',
  author_email = 'olivier.borderies@gmail.com',
  url = 'https://gitlab.com/oscar6echo/ezchat',
  download_url = 'https://gitlab.com/oscar6echo/ezchat/repository/archive.tar.gz?ref=' + version,
  keywords = ['chat', 'notebook', 'javascript', 'vuejs'],
  license='MIT',
  classifiers = [ 'Development Status :: 4 - Beta',
                  'License :: OSI Approved :: MIT License',
                  'Programming Language :: Python :: 3.5',
                  'Programming Language :: Python :: 3.6'
  ],
  install_requires = [
    'Flask>=0.12.2',
    'Flask-SocketIO>=2.9.1'
  ],
  include_package_data=True,
  package_data={
    'templates':
         ['templates/info.html'
         ],
    'chatbox':
        ['chatbox/src/components/App/component.html',
         'chatbox/src/components/App/component.js',
         'chatbox/src/components/App/component.less',
         'chatbox/src/components/App/component.vue',
         'chatbox/src/components/ChatBox/component.html',
         'chatbox/src/components/ChatBox/component.js',
         'chatbox/src/components/ChatBox/component.less',
         'chatbox/src/components/ChatBox/component.vue',
         'chatbox/src/components/ChatLine/component.html',
         'chatbox/src/components/ChatLine/component.js',
         'chatbox/src/components/ChatLine/component.less',
         'chatbox/src/components/ChatLine/component.vue',
         'chatbox/src/css/main.less',
         'chatbox/src/css/variables.less',
         'chatbox/src/img/favicon.ico',
         'chatbox/src/main.js',
         'chatbox/src/top.html',
         'chatbox/.babelrc',
         'chatbox/index.html',
         'chatbox/package.json',
         'chatbox/README.md',
         'chatbox/webpack.config.js',
         'chatbox/dist/index.html',
         'chatbox/dist/assets/app.js',
         'chatbox/dist/assets/favicon.ico',
         'chatbox/dist/assets/style/app.css',
         'chatbox/dist/assets/style/app-less.css',
         'chatbox/dist/assets/font/fontawesome-webfont.eot',
         'chatbox/dist/assets/font/fontawesome-webfont.svg',
         'chatbox/dist/assets/font/fontawesome-webfont.ttf',
         'chatbox/dist/assets/font/fontawesome-webfont.woff',
         'chatbox/dist/assets/font/fontawesome-webfont.woff2',
         'chatbox/dist/assets/font/glyphicons-halflings-regular.eot',
         'chatbox/dist/assets/font/glyphicons-halflings-regular.svg',
         'chatbox/dist/assets/font/glyphicons-halflings-regular.ttf',
         'chatbox/dist/assets/font/glyphicons-halflings-regular.woff',
         'chatbox/dist/assets/font/glyphicons-halflings-regular.woff2'
        ]
    },
)

